SystemOrganization addCategory: #'Neo-JSON-Core'!

!Collection methodsFor: '*neo-json-core' stamp: 'SvenVanCaekenberghe 6/3/2012 19:29'!
neoJsonOn: neoJSONWriter
	neoJSONWriter writeList: self! !

!Number methodsFor: '*neo-json-core' stamp: 'SvenVanCaekenberghe 6/2/2012 22:17'!
neoJsonOn: neoJSONWriter
	neoJSONWriter writeFloat: self asFloat! !

NotFound subclass: #NeoJSONMappingNotFound
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Core'!

!NeoJSONMappingNotFound commentStamp: '<historical>' prior: 0!
I am NeoJSONMappingNotFound.
I am a NotFound exception.

I am signalled when no mapping is found for a schema.!

!NeoJSONMappingNotFound methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/12/2012 15:47'!
standardMessageText
	^ String streamContents: [ :stream |
		stream << 'No mapping found for '.
		stream print: self object.
		stream << ' in '.
		stream print: self collection class ]! !

!Dictionary methodsFor: '*neo-json-core' stamp: 'SvenVanCaekenberghe 6/3/2012 19:30'!
neoJsonOn: neoJSONWriter
	neoJSONWriter writeMap: self! !

!UndefinedObject methodsFor: '*neo-json-core' stamp: 'SvenVanCaekenberghe 6/2/2012 22:15'!
neoJsonOn: neoJSONWriter
	neoJSONWriter writeNull! !

!SmallDictionary methodsFor: '*neo-json-core' stamp: 'SvenVanCaekenberghe 6/3/2012 19:30'!
neoJsonOn: neoJSONWriter
	neoJSONWriter writeMap: self! !

!Integer methodsFor: '*neo-json-core' stamp: 'SvenVanCaekenberghe 6/2/2012 22:17'!
neoJsonOn: neoJSONWriter
	neoJSONWriter writeInteger: self! !

!Boolean methodsFor: '*neo-json-core' stamp: 'SvenVanCaekenberghe 6/2/2012 22:14'!
neoJsonOn: neoJSONWriter
	neoJSONWriter writeBoolean: self! !

Object subclass: #NeoJSONMapper
	instanceVariableNames: 'mappings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Core'!

!NeoJSONMapper commentStamp: '<historical>' prior: 0!
I am NeoJSONMapper.
I hold a mapping from schema identifiers to objects that known how to read or write instances from or to JSON.

Schema identifiers are either Smalltalk classes or Symbols.

I offer a builder interface to construct mappings. 

The standard object mapping represents a Smalltalk object as a map, mapping some or all instance variables, or a set of accessors (getter/setter methods). Furthermore, for each value an optional schema can be specified.

Using a custom mapping, a reader and/or writer block can deal with special cases such as specific collection types with an optional schema for the elements, or a direct mapping of semi primitive types such as Date or DateAndTime.

A mapping can be specified explicitely on a mapper, or can be resolved using the #neoJsonMapping: class method.

Here are some examples:

	mapper mapAllInstVarsFor: Point.
	mapper for: TestObject do: [ :mapping |
		mapping mapInstVars: #(id name).
		(mapping mapInstVar: #timestamp to: 'created-at') valueSchema: DateAndTime.
		(mapping mapInstVar: #points) valueSchema: #ArrayOfPoints.
		(mapping mapInstVar: #bytes) valueSchema: ByteArray ].
	mapper for: DateAndTime customDo: [ :mapping |
		mapping decoder: [ :string | DateAndTime fromString: string ].
		mapping encoder: [ :dateAndTime | dateAndTime printString ] ].
	mapper for: #ArrayOfPoints customDo: [ :mapping |
		mapping listOfElementSchema: Point ].  
	mapper for: ByteArray customDo: [ :mapping |
		mapping listOfType: ByteArray ]
!

!NeoJSONMapper methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/22/2014 10:57'!
customMappingFor: schemaName
	^ self mappings
		at: schemaName
		ifAbsentPut: [ 
			NeoJSONCustomMapping new
				schemaName: schemaName;
				yourself ]! !

!NeoJSONMapper methodsFor: 'mapping' stamp: 'SvenVanCaekenberghe 3/22/2014 10:56'!
for: schemaName customDo: block
	"Execute the code in block to further customize the mapping for schemaName.
	Create and add a new custom mapping for schemaName if necessary."
	
	| mapping |
	mapping := self customMappingFor: schemaName.
	block value: mapping.
	^ mapping! !

!NeoJSONMapper methodsFor: 'mapping' stamp: 'SvenVanCaekenberghe 3/22/2014 10:57'!
for: smalltalkClass do: block
	"Execute the code in block to further customize the mapping for schemaName.
	Create and add a new standard object mapping for schemaName if necessary."
	
	| mapping |
	mapping := self objectMappingFor: smalltalkClass.
	block value: mapping.
	^ mapping! !

!NeoJSONMapper methodsFor: 'convencience' stamp: 'SvenVanCaekenberghe 3/22/2014 10:54'!
mapAllInstVarsFor: smalltalkClass
	"Map the instance variables of smalltalkClass and all its super classes"

	| mapping |
	mapping := self objectMappingFor: smalltalkClass.
	mapping mapAllInstVars.
	^ mapping! !

!NeoJSONMapper methodsFor: 'convencience' stamp: 'SvenVanCaekenberghe 3/22/2014 10:54'!
mapInstVarsFor: smalltalkClass
	"Map the instance variables of smalltalkClass"

	| mapping |
	mapping := self objectMappingFor: smalltalkClass.
	mapping mapInstVars.
	^ mapping! !

!NeoJSONMapper methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/12/2012 15:48'!
mappingFor: schema
	^ self 
		mappingFor: schema 
		ifAbsent: [ NeoJSONMappingNotFound signalFor: schema in: self ]! !

!NeoJSONMapper methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/9/2012 18:24'!
mappingFor: schema ifAbsent: block
	(self mappings includesKey: schema)
		ifFalse: [ self resolveMappingFor: schema ].
	^ self mappings
		at: schema
		ifAbsent: block! !

!NeoJSONMapper methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/7/2012 14:55'!
mappings
	^ mappings
		ifNil: [ mappings := Dictionary new ]! !

!NeoJSONMapper methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/22/2014 10:54'!
objectMappingFor: smalltalkClass
	^ self mappings
		at: smalltalkClass
		ifAbsentPut: [ 
			NeoJSONObjectMapping new
				subjectClass: smalltalkClass;
				yourself ]! !

!NeoJSONMapper methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/9/2012 18:29'!
resolveMappingFor: class
	(class respondsTo: #neoJsonMapping:)
		ifTrue: [ 
			class perform: #neoJsonMapping: with: self ]! !

NeoJSONMapper subclass: #NeoJSONReader
	instanceVariableNames: 'readStream stringStream listClass mapClass propertyNamesAsSymbols'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Core'!

!NeoJSONReader commentStamp: '<historical>' prior: 0!
I am NeoJSONReader.
I am a NeoJSONMapper.
I parse a JSON representation from a textual stream into Smalltalk objects.

Primitive types are automatically mapped to their Smalltalk counterpart: Numbers, Strings, Booleans and UndefinedObject.
When parsing using #next, lists normally become Arrays and maps become Dictionaries.
An alternative listClass or mapClass can be specified.

When parsing using #nextAs: a schema can be specified. This schema defines a mapping to be used.

Here are some examples:

	NeoJSONReader fromString: ' [ 1,2,3 ] '.
	NeoJSONReader fromString: ' [ 3.14159, true, false, null, "string" ] '.
	NeoJSONReader fromString: ' { "x" : 1, "y" : 2 } '.
	
	(NeoJSONReader on: ' { "x" : 1, "y" : 2 } ' readStream)
		mapInstVarsFor: Point;
		nextAs: Point.	
			
	(NeoJSONReader on: '[ { "x" : 1, "y" : 2 }, { "x" : 3, "y" : 4 } ]' readStream)
		mapInstVarsFor: Point;
		for: #ArrayOfPoints customDo: [ :mapping |
			mapping listOfElementSchema: Point ];
		nextAs: #ArrayOfPoints.!

!NeoJSONReader class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/9/2012 18:26'!
fromString: string
	^ (self on: string readStream) next! !

!NeoJSONReader class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/9/2012 18:30'!
fromString: string as: schema
	^ (self on: string readStream) nextAs: schema! !

!NeoJSONReader class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 6/13/2012 16:10'!
on: readStream
	"Initialize on readStream, which should be a character stream that 
	implements #next, #peek, #atEnd and (optionally) #close."

	^ self new
		on: readStream;
		yourself! !

!NeoJSONReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 3/6/2013 14:46'!
atEnd
	^ readStream atEnd! !

!NeoJSONReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/2/2012 13:50'!
close
	readStream ifNotNil: [
		readStream close.
		readStream := nil ]! !

!NeoJSONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/2/2012 21:26'!
consumeWhitespace
	"Strip whitespaces from the input stream."

	[ readStream atEnd not and: [ readStream peek isSeparator ] ]
		whileTrue: [ 
			readStream next ]! !

!NeoJSONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/12/2012 15:54'!
error: text
	NeoJSONParseError signal: text! !

!NeoJSONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/2/2012 14:01'!
expectChar: character
	"Expect character and consume input and optional whitespace at the end,
	 throw an error otherwise."

	(self matchChar: character)
		ifFalse: [ self error: character asString, ' expected' ]! !

!NeoJSONReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/13/2012 09:39'!
initialize 
	super initialize.
	propertyNamesAsSymbols := false! !

!NeoJSONReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2012 09:37'!
listClass
	"Return the collection type to use for JSON lists"
	
	^ listClass ifNil: [ Array ]! !

!NeoJSONReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/13/2012 09:38'!
listClass: collectionClass
	listClass := collectionClass! !

!NeoJSONReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2012 09:37'!
mapClass
	"Return the collection type to use for JSON maps"
	
	^ mapClass ifNil: [ Dictionary ]! !

!NeoJSONReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/13/2012 09:38'!
mapClass: collectionClass
	mapClass := collectionClass! !

!NeoJSONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/3/2012 10:57'!
match: string do: block
	"Try to read and consume string and execute block if successful.
	Else do nothing (but do not back up)"

	(string allSatisfy: [ :each | readStream peekFor: each ])
		ifTrue: [ 
			self consumeWhitespace.
			block value ]! !

!NeoJSONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/3/2012 10:51'!
matchChar: character
	"Tries to match character, consume input and 
	answer true if successful and consumes whitespace at the end.
	When character does not match, return false and leave stream untouched."

	^ (readStream peekFor: character)
		ifTrue: [ 
			self consumeWhitespace.
			true ]
		ifFalse: [ false ]! !

!NeoJSONReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/7/2012 16:56'!
next
	"Primary interface to parse a JSON value.
	Return either primitives, a listClass or a mapClass instance."
	
	^ self 
		consumeWhitespace; 
		parseValue! !

!NeoJSONReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/7/2012 22:21'!
nextAs: schema
	"Secondary interface to parse JSON.
	Return objects according to schema."
	
	| mapping |
	schema ifNil: [ ^ self next ].
	self consumeWhitespace.
	mapping := self mappingFor: schema.
	^ mapping readFrom: self! !

!NeoJSONReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/13/2012 16:10'!
on: aReadStream
	"Initialize on aReadStream, which should be a character stream that 
	implements #next, #peek, #atEnd and (optionally) #close."
	
	readStream := aReadStream! !

!NeoJSONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 6/7/2012 16:59'!
parseCharacter
	| char |
	(char := readStream next) = $\ 
		ifFalse: [ ^ char ].
	('/"\' includes: (char := readStream next))
		ifTrue: [ ^ char ].
	('bfnrt' includes: char)
		ifTrue: [ ^ self unescapeChar: char ].
	char = $u
		ifTrue: [ ^ self parseCharacterHex ].
	self error: 'invalid escape character \' , (String with: char)! !

!NeoJSONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 6/2/2012 14:20'!
parseCharacterHex
	| value |
	value := self parseCharacterHexDigit.
	3 timesRepeat: [ 
		value := (value << 4) + self parseCharacterHexDigit ].
	^ Character codePoint: value! !

!NeoJSONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 6/2/2012 14:21'!
parseCharacterHexDigit
	| digit |
	readStream atEnd 
		ifFalse: [ 
			digit := readStream next asInteger.
			(digit between: "$0" 48 and: "$9" 57)
				ifTrue: [ ^ digit - 48 ].
			(digit between: "$A" 65 and: "$F" 70)
				ifTrue: [ ^ digit - 55 ].
			(digit between: "$a" 97 and: "$f" 102)
				ifTrue: [ ^ digit - 87 ] ].
	self error: 'hex-digit expected'! !

!NeoJSONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/2/2012 14:16'!
parseConstantDo: block
	"Parse and consume either true|false|nil|null and execute block 
	or else do nothing (but do not back up).
	Hand written implementation to avoid the use of #position:"
	
	(readStream peek = $t)
		ifTrue: [
			^ self match: 'true' do: [ block value: true ] ].
	(readStream peek = $f)
		ifTrue: [
			^ self match: 'false' do: [ block value: false ] ].
	(readStream peek = $n)
		ifTrue: [
			^ self match: 'null' do: [ block value: nil ] ]
! !

!NeoJSONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 6/7/2012 17:17'!
parseList
	^ self listClass 
		streamContents: [ :stream |
			self parseListElementsDo: [ :each |
				stream nextPut: each ] ]

		! !

!NeoJSONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 6/7/2012 17:15'!
parseListDo: block
	self expectChar: $[.
	(self matchChar: $]) 
		ifTrue: [ ^ self ].
	[ readStream atEnd ] 
		whileFalse: [
			block value.
			(self matchChar: $]) 
				ifTrue: [ ^ self ].
			self expectChar: $, ].
	self error: 'end of list expected'! !

!NeoJSONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 6/7/2012 17:16'!
parseListElementsDo: block
	self parseListDo: [ 
		block value: self parseValue ]! !

!NeoJSONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 6/7/2012 17:23'!
parseMap
	| map |
	map := self mapClass new.
	self parseMapKeysAndValuesDo: [ :key :value |
		map at: key put: value ].
	^ map! !

!NeoJSONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 6/7/2012 17:23'!
parseMapDo: block
	self expectChar: ${.
	(self matchChar: $})
		ifTrue: [ ^ self ].
	[ readStream atEnd ] whileFalse: [
		block value.
		(self matchChar: $}) 
			ifTrue: [ ^ self ].
		self expectChar: $, ].
	self error: 'end of map expected'! !

!NeoJSONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 6/7/2012 17:22'!
parseMapKeysAndValuesDo: block
	self parseMapKeysDo: [ :key |
		block value: key value: self parseValue ]! !

!NeoJSONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 6/7/2012 17:23'!
parseMapKeysDo: block
	self parseMapDo: [ | key |
		key := self parsePropertyName.
		self expectChar: $:.
		block value: key ]! !

!NeoJSONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 12/3/2012 11:08'!
parseNumber
	| negated number |
	negated := readStream peekFor: $-.
	number := self parseNumberInteger.
	(readStream peekFor: $.)
		ifTrue: [ number := number + self parseNumberFraction ].
	((readStream peekFor: $e) or: [ readStream peekFor: $E ])
		ifTrue: [ number := number * self parseNumberExponent ].
	negated
		ifTrue: [ number := number negated ].
	self consumeWhitespace.
	^ number! !

!NeoJSONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 12/3/2012 11:08'!
parseNumberExponent
	| number negated |
	number := 0.
	(negated := readStream peekFor: $-)
		ifFalse: [ readStream peekFor: $+ ].
	[ readStream atEnd not and: [ readStream peek isDigit ] ]
		whileTrue: [ number := 10 * number + readStream next digitValue ].
	negated
		ifTrue: [ number := number negated ].
	^ 10 raisedTo: number! !

!NeoJSONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 6/2/2012 14:00'!
parseNumberFraction
	| number power |
	number := 0.
	power := 1.0.
	[ readStream atEnd not and: [ readStream peek isDigit ] ] 
		whileTrue: [
			number := 10 * number + readStream next digitValue.
			power := power * 10.0 ].
	^ number / power! !

!NeoJSONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 6/12/2012 16:15'!
parseNumberInteger
	| number |
	number := nil.
	[ readStream atEnd not and: [ readStream peek isDigit ] ] 
		whileTrue: [ 
			number := 10 * (number ifNil: [ 0 ]) + readStream next digitValue ].
	number ifNil: [ self error: 'Integer digit expected' ].
	^ number! !

!NeoJSONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 6/13/2012 09:40'!
parsePropertyName
	| name |
	name := self parseValue.
	^ name isString
		ifTrue: [ 
			propertyNamesAsSymbols 
				ifTrue: [ name asSymbol ]
				ifFalse: [ name ] ]
		ifFalse: [ 
			self error: 'property name must be a string, not ', name asString ]! !

!NeoJSONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 6/2/2012 13:58'!
parseString
	| result |
	self expectChar: $".
	result := self stringStreamContents: [ :stream |
		[ readStream atEnd or: [ readStream peek = $"] ] 
			whileFalse: [ 
				stream nextPut: self parseCharacter ] ].
	self expectChar: $".
	^ result! !

!NeoJSONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 6/12/2012 15:56'!
parseValue
	| char |
	readStream atEnd ifFalse: [ 
		char := readStream peek.
		char = ${
			ifTrue: [ ^ self parseMap ].
		char = $[
			ifTrue: [ ^ self parseList ].
		char = $"
			ifTrue: [ ^ self parseString ].
		(char = $- or: [ char isDigit ])
			ifTrue: [ ^ self parseNumber ].
		self parseConstantDo: [ :value | ^ value ] ].
	self error: 'invalid input: ', char asString! !

!NeoJSONReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/13/2012 09:39'!
propertyNamesAsSymbols: boolean
	propertyNamesAsSymbols := boolean! !

!NeoJSONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/7/2012 16:54'!
stringStreamContents: block
	"Like String streamContents: block
	but reusing the underlying buffer for improved efficiency"
	
	stringStream 
		ifNil: [ 
			stringStream := (String new: 32) writeStream ].
	stringStream reset.
	block value: stringStream.
	^ stringStream contents! !

!NeoJSONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/7/2012 16:55'!
unescapeChar: char
	"Handle one of the named character escapes, b, f, n, r or t. Else fail."
	
	char = $b
		ifTrue: [ ^ Character backspace ].
	char = $f
		ifTrue: [ ^ Character newPage ].
	char = $n
		ifTrue: [ ^ Character lf ].
	char = $r
		ifTrue: [ ^ Character cr ].
	char = $t
		ifTrue: [ ^ Character tab ].
	self error: 'Unknown escape char: ', (String with: char)! !

NeoJSONMapper subclass: #NeoJSONWriter
	instanceVariableNames: 'writeStream prettyPrint level newLine asciiOnly writeNil'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Core'!

!NeoJSONWriter commentStamp: '<historical>' prior: 0!
I am NeoJSONWriter.
I am a NeoJSONMapper.
I write a JSON representation of Smalltalk objects to a textual stream.

Objects implementing #neoJsonOn: can be encoded.
A number of primitive types, like Strings, Numbers, Booleans and UndefinedObject are treated specially.
Collection are encoded as lists, except for Dictionaries that are encoded as maps.

All other objects need a mapping to help in doing the encoding.

Here are some examples:

	NeoJSONWriter toString: #(1 2 3).
	NeoJSONWriter toString: { Float pi. true. false. 'string' }.
	NeoJSONWriter toStringPretty: (Dictionary new at: #x put: 1; at: #y put: 2; yourself).
	
	String streamContents: [ :stream |
		(NeoJSONWriter on: stream)
			prettyPrint: true;
			mapInstVarsFor: Point;
			nextPut: (Array with: 1@3 with: -1@3) ].
	!

!NeoJSONWriter class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 6/14/2012 09:54'!
on: writeStream
	"Initialize on writeStream, which should be a character stream that 
	implements #nextPut:, #nextPutAll:, #space and (optionally) #close."

	^ self new
		on: writeStream;
		yourself! !

!NeoJSONWriter class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/9/2012 18:27'!
toString: object
	^ String streamContents: [ :stream |
			(self on: stream) nextPut: object ]! !

!NeoJSONWriter class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/9/2012 18:28'!
toStringPretty: object
	^ String streamContents: [ :stream |
			(self on: stream)
				prettyPrint: true; 
				nextPut: object ]! !

!NeoJSONWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/3/2013 12:00'!
asciiOnly: boolean
	asciiOnly := boolean! !

!NeoJSONWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/2/2012 22:06'!
close
	writeStream ifNotNil: [
		writeStream close.
		writeStream := nil ]! !

!NeoJSONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/3/2013 11:58'!
doesCodePointNeedEscaping: code
	code < 32 ifTrue: [ ^ true ].
	( code = 34 or: [ code = 92 ]) ifTrue: [ ^ true ].
	^ asciiOnly and: [ code > 126 ]	! !

!NeoJSONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/3/2013 11:55'!
encodeChar: char
	| code |
	code := char codePoint.
	(self doesCodePointNeedEscaping: code)
		ifTrue: [ self escapeChar: code ]
		ifFalse: [ writeStream nextPut: char ]! !

!NeoJSONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/3/2012 19:41'!
encodeKey: key value: value
	self nextPut: key.
	self prettyPrintSpace.
	writeStream nextPut: $:.
	self prettyPrintSpace.
	self nextPut: value! !

!NeoJSONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/23/2014 21:53'!
encodeKey: key value: value as: valueSchema
	self nextPut: key.
	self prettyPrintSpace.
	writeStream nextPut: $:.
	self prettyPrintSpace.
	self nextPut: value as: valueSchema! !

!NeoJSONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/2/2012 22:45'!
escapeChar: code
	code = 34
		ifTrue: [ ^ writeStream nextPutAll: '\"' ].
	code = 92
		ifTrue: [ ^ writeStream nextPutAll: '\\' ].
	code = 47
		ifTrue: [ ^ writeStream nextPutAll: '\/' ].
	code = 8
		ifTrue: [ ^ writeStream nextPutAll: '\b' ].
	code = 12
		ifTrue: [ ^ writeStream nextPutAll: '\f' ].		
	code = 10
		ifTrue: [ ^ writeStream nextPutAll: '\n' ].		
	code = 13
		ifTrue: [ ^ writeStream nextPutAll: '\r' ].		
	code = 9
		ifTrue: [ ^ writeStream nextPutAll: '\t' ].		
	writeStream nextPutAll: '\u'.
	code printOn: writeStream base: 16 nDigits: 4! !

!NeoJSONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/3/2012 19:31'!
indentedDo: block
	level := level + 1.
	block value.
	level := level - 1! !

!NeoJSONWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/28/2014 20:43'!
initialize
	super initialize.
	self newLine: String cr.
	self prettyPrint: false.
	self asciiOnly: false.
	self writeNil: false.
	level := 0! !

!NeoJSONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/3/2012 19:32'!
listElementSeparator
	writeStream nextPut: $,.
	self newlineIndent
! !

!NeoJSONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/3/2012 19:32'!
mapElementSeparator
	writeStream nextPut: $,.
	self newlineIndent
! !

!NeoJSONWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 12/10/2012 16:27'!
newLine: string
	newLine := string! !

!NeoJSONWriter methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/17/2014 11:58'!
newline
	"Write a newline on the stream that I wrap.
	What gets written depends on the configuration, see #newLine:"
	
	writeStream nextPutAll: newLine! !

!NeoJSONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/17/2014 11:55'!
newlineIndent
	prettyPrint
		ifTrue: [ 
			self newline.
			level timesRepeat: [ writeStream nextPut: Character tab ] ]! !

!NeoJSONWriter methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/2/2012 22:11'!
nextPut: anObject
	anObject neoJsonOn: self! !

!NeoJSONWriter methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/23/2014 21:57'!
nextPut: anObject as: schema
	"Secondary interface to write JSON.
	Write objects according to schema."
	
	| mapping |
	schema ifNil: [ ^ self nextPut: anObject ].
	mapping := self mappingFor: schema.
	^ mapping writeObject: anObject on: self! !

!NeoJSONWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/14/2012 09:54'!
on: aWriteStream
	"Initialize on aWriteStream, which should be a character stream that 
	implements #nextPut:, #nextPutAll:, #space and (optionally) #close."

	writeStream := aWriteStream
! !

!NeoJSONWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/2/2012 22:06'!
prettyPrint: boolean
	prettyPrint := boolean! !

!NeoJSONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/10/2012 16:33'!
prettyPrintSpace
	prettyPrint
		ifTrue: [ writeStream nextPut: Character space ]! !

!NeoJSONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 6/2/2012 22:18'!
writeBoolean: boolean
	boolean printOn: writeStream! !

!NeoJSONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 6/2/2012 22:19'!
writeFloat: float
	float printOn: writeStream! !

!NeoJSONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 6/2/2012 22:18'!
writeInteger: integer
	integer printOn: writeStream
	! !

!NeoJSONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 6/6/2012 15:20'!
writeList: collection
	self writeListStreamingDo: [ :jsonListWriter |
		collection do: [ :each |
			jsonListWriter writeElement: each ] ]! !

!NeoJSONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 6/6/2012 15:19'!
writeListStreamingDo: block
	writeStream nextPut: $[.
	self writeStreamingDo: block.
	writeStream nextPut: $]! !

!NeoJSONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 6/6/2012 15:04'!
writeMap: keyValueCollection
	self writeMapStreamingDo: [ :jsonMapWriter |
		keyValueCollection keysAndValuesDo: [ :key :value |
			jsonMapWriter writeKey: key value: value ] ]! !

!NeoJSONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 6/6/2012 15:19'!
writeMapStreamingDo: block
	writeStream nextPut: ${.
	self writeStreamingDo: block.
	writeStream nextPut: $}! !

!NeoJSONWriter methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/28/2014 20:44'!
writeNil
	^ writeNil! !

!NeoJSONWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/28/2014 20:43'!
writeNil: boolean
	writeNil := boolean! !

!NeoJSONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 6/2/2012 22:15'!
writeNull
	writeStream nextPutAll: 'null'! !

!NeoJSONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 6/7/2012 15:55'!
writeObject: anObject
	| mapping |
	mapping := self mappingFor: anObject class.
	mapping
		writeObject: anObject
		on: self! !

!NeoJSONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 6/6/2012 15:19'!
writeStreamingDo: block
	| mapWriter |
	mapWriter := NeoJSONStreamingWriter on: self.
	self indentedDo: [
		block value: mapWriter ].
	mapWriter wasUsed
		ifTrue: [ self newlineIndent ]
		ifFalse: [ self prettyPrintSpace ]! !

!NeoJSONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 6/3/2012 19:42'!
writeString: string
	writeStream nextPut: $".
	string do: [ :each |
		self encodeChar: each ].
	writeStream nextPut: $"! !

Object subclass: #NeoJSONMapping
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Core'!

!NeoJSONMapping commentStamp: 'SvenVanCaekenberghe 3/23/2014 19:22' prior: 0!
I am NeoJSONMapping, an abstract superclass for mapping definitions.
!

NeoJSONMapping subclass: #NeoJSONCustomMapping
	instanceVariableNames: 'schemaName reader writer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Core'!

!NeoJSONCustomMapping commentStamp: '<historical>' prior: 0!
I am NeoJSONCustomMapping.

I read the next JSON value and turn it into a Smalltalk object using a converter block.!

!NeoJSONCustomMapping methodsFor: 'mapping' stamp: 'SvenVanCaekenberghe 6/13/2012 11:14'!
decoder: block
	"Set a decoder block that accepts the next object representation read 
	and converts it into the actual object to be returned."

	self reader: [ :jsonReader | | representation |
		representation := jsonReader next.
		block value: representation ]! !

!NeoJSONCustomMapping methodsFor: 'mapping' stamp: 'SvenVanCaekenberghe 6/13/2012 11:14'!
encoder: block
	"Set a encoder block that accepts the next object to be written 
	and converts it into the actual representation to be written."

	self writer: [ :jsonWriter :anObject | | representation |
		representation := block value: anObject.
		jsonWriter nextPut: representation ]! !

!NeoJSONCustomMapping methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/23/2014 19:24'!
identifier
	^ schemaName ! !

!NeoJSONCustomMapping methodsFor: 'mapping' stamp: 'SvenVanCaekenberghe 6/13/2012 11:38'!
listOfElementSchema: elementSchema
	"Set the reader to a block that will create an instance of the
	standard listClass while reading a list of elements of type elementSchema"

	self reader: [ :jsonReader |
		jsonReader listClass streamContents: [ :stream |
			jsonReader parseListDo: [ | element |
				element := jsonReader nextAs: elementSchema.
				stream nextPut: element ] ] ]
	"we do not set a writer, #neoJsonOn: should be sufficient"
! !

!NeoJSONCustomMapping methodsFor: 'mapping' stamp: 'SvenVanCaekenberghe 6/13/2012 11:25'!
listOfType: collectionClass
	"Set the reader to a block that will create an instance of collectionClass 
	using #streamContents: while reading a list of elements"

	self reader: [ :jsonReader |
		collectionClass streamContents: [ :stream |
			jsonReader parseListDo: [ | element |
				element := jsonReader next.
				stream nextPut: element ] ] ]
	"we do not set a writer, #neoJsonOn: should be sufficient"
! !

!NeoJSONCustomMapping methodsFor: 'mapping' stamp: 'SvenVanCaekenberghe 6/13/2012 11:39'!
listOfType: collectionClass andElementSchema: elementSchema
	"Set the reader to a block that will create an instance of collectionClass 
	using #streamContents: while reading a list of elements of type elementSchema"

	self reader: [ :jsonReader |
		collectionClass streamContents: [ :stream |
			jsonReader parseListDo: [ | element |
				element := jsonReader nextAs: elementSchema.
				stream nextPut: element ] ] ]
	"we do not set a writer, #neoJsonOn: should be sufficient"
! !

!NeoJSONCustomMapping methodsFor: 'mapping' stamp: 'SvenVanCaekenberghe 8/30/2013 11:04'!
mapWithValueSchema: valueSchema
	"Set the reader to a block that will create an instance of the
	standard mapClass while reading values of type valueSchema,
	with standard keys"

	self reader: [ :jsonReader | | map |
		map := jsonReader mapClass new.
		jsonReader parseMapKeysDo: [ :key | | value |
				value := jsonReader nextAs: valueSchema.
				map at: key put: value ].
		map ]
	"we do not set a writer, #neoJsonOn: should be sufficient"
! !

!NeoJSONCustomMapping methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 3/24/2014 10:34'!
readFrom: jsonReader
	^ reader 
		ifNil: [ jsonReader next ]
		ifNotNil: [ reader value: jsonReader ]! !

!NeoJSONCustomMapping methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/24/2014 10:36'!
reader: block
	"Set the reader to block, a 1 argument block that when given 
	a jsonReader as argument reads the object that I represent.
	When there is no reader block, the default #next will be used."
	
	reader := block! !

!NeoJSONCustomMapping methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/8/2012 21:09'!
schemaName: schema
	schemaName := schema! !

!NeoJSONCustomMapping methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 3/23/2014 22:10'!
writeObject: anObject on: jsonWriter
	writer 
		ifNil: [ anObject neoJsonOn: jsonWriter ]
		ifNotNil: [ writer value: jsonWriter value: anObject ]! !

!NeoJSONCustomMapping methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/23/2014 22:12'!
writer: block
	"Set the writer to block, a 2 argument block that when given: a jsonWriter 
	and the object to write will write the object that I represent.
	When there is no writer block, the default #neoJsonOn: will be used."
	
	writer := block! !

!NeoJSONMapping methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/23/2014 19:22'!
identifier
	self subclassResponsibility ! !

!NeoJSONMapping methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 3/23/2014 19:24'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(; print: self identifier; nextPut: $)! !

!NeoJSONMapping methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 3/23/2014 19:23'!
readFrom: jsonReader
	self subclassResponsibility ! !

!NeoJSONMapping methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 3/23/2014 19:23'!
writeObject: anObject on: jsonWriter
	self subclassResponsibility ! !

NeoJSONMapping subclass: #NeoJSONObjectMapping
	instanceVariableNames: 'subjectClass properties'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Core'!

!NeoJSONObjectMapping commentStamp: '<historical>' prior: 0!
I am NeoJSONObjectMapping.
I describe how instances of a Smalltalk class are mapped to a JSON map.

I do this using set of NeoJSONPropertyMappings.

I know the Smalltalk class that I am a mapping for. This class is the schema identifier and is used to instanciate new objects.

A mapping can be based on any combination of instance variables or accessors (getter/setter pairs). 

I offer a builder interface to constuct property mappings.!

!NeoJSONObjectMapping methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/23/2014 19:25'!
identifier
	^ subjectClass ! !

!NeoJSONObjectMapping methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/5/2012 22:50'!
initialize
	super initialize.
	properties := Dictionary new.! !

!NeoJSONObjectMapping methodsFor: 'mapping' stamp: 'SvenVanCaekenberghe 6/12/2012 13:13'!
mapAccessor: accessor
	^ self mapAccessor: accessor to: accessor ! !

!NeoJSONObjectMapping methodsFor: 'mapping' stamp: 'SvenVanCaekenberghe 6/12/2012 13:12'!
mapAccessor: accessor mutator: mutator to: propertyName
	^ self 
		mapProperty: propertyName
		getter: [ :object | object perform: accessor ]
		setter: [ :object :value | object perform: mutator with: value ] ! !

!NeoJSONObjectMapping methodsFor: 'mapping' stamp: 'SvenVanCaekenberghe 6/12/2012 13:13'!
mapAccessor: accessor to: propertyName
	^ self 
		mapProperty: propertyName
		getter: [ :object | object perform: accessor ]
		setter: [ :object :value | object perform: accessor asMutator with: value ] ! !

!NeoJSONObjectMapping methodsFor: 'mapping' stamp: 'SvenVanCaekenberghe 6/12/2012 13:14'!
mapAccessors: accessors
	accessors do: [ :each |
		self mapAccessor: each ]! !

!NeoJSONObjectMapping methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/6/2012 16:00'!
mapAllInstVars
	self mapInstVars: subjectClass allInstVarNames! !

!NeoJSONObjectMapping methodsFor: 'mapping' stamp: 'SvenVanCaekenberghe 6/9/2012 18:37'!
mapInstVar: instVarName
	^ self mapInstVar: instVarName to: instVarName ! !

!NeoJSONObjectMapping methodsFor: 'mapping' stamp: 'SvenVanCaekenberghe 6/9/2012 18:36'!
mapInstVar: instVarName to: propertyName
	| index |
	index := subjectClass 
		instVarIndexFor: instVarName 
		ifAbsent: [ self error: 'Inst var not found: ', instVarName printString ].
	^ self 
		mapProperty: propertyName
		getter: [ :object | object instVarAt: index ]
		setter: [ :object :value | object instVarAt: index put: value ] ! !

!NeoJSONObjectMapping methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/6/2012 15:59'!
mapInstVars
	self mapInstVars: subjectClass instVarNames! !

!NeoJSONObjectMapping methodsFor: 'mapping' stamp: 'SvenVanCaekenberghe 6/6/2012 16:00'!
mapInstVars: instVarNames
	instVarNames do: [ :each |
		self mapInstVar: each ]! !

!NeoJSONObjectMapping methodsFor: 'mapping' stamp: 'SvenVanCaekenberghe 6/9/2012 18:13'!
mapProperty: propertyName getter: readBlock setter: writeBlock
	| propertyMapping |
	propertyMapping := NeoJSONPropertyMapping new.
	propertyMapping 
		propertyName: propertyName;
		getter: readBlock;
		setter: writeBlock.
	properties at: propertyName put: propertyMapping.
	^ propertyMapping ! !

!NeoJSONObjectMapping methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/7/2012 14:52'!
propertyNamed: propertyName ifAbsent: block
	^ properties at: propertyName ifAbsent: block! !

!NeoJSONObjectMapping methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 6/8/2012 17:13'!
readFrom: jsonReader
	| anObject |
	anObject := subjectClass new.
	jsonReader parseMapKeysDo: [ :key |
		(self propertyNamed: key ifAbsent: [ nil ])
			ifNil: [
				"read, skip & ignore value"
				jsonReader next ]
			ifNotNil: [ :mapping |
				mapping readObject: anObject from: jsonReader ] ].
	^ anObject! !

!NeoJSONObjectMapping methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/6/2012 15:32'!
subjectClass: smalltalkClass
	subjectClass := smalltalkClass! !

!NeoJSONObjectMapping methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 6/6/2012 14:55'!
writeObject: anObject on: jsonWriter
	jsonWriter writeMapStreamingDo: [ :jsonMapWriter |
		properties valuesDo: [ :each |
			each writeObject: anObject on: jsonMapWriter ] ]! !

Object subclass: #NeoJSONPropertyMapping
	instanceVariableNames: 'propertyName valueSchema getter setter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Core'!

!NeoJSONPropertyMapping commentStamp: '<historical>' prior: 0!
I am NeoJSONPropertyMapping.
I describe how a property of a Smalltalk class maps to a JSON property.

A property is either an instance variable or getter/setter pair.
I hold a getter and setter block to do the actual work. The property name is used in JSON.

Optionally, the schema for the value of my property can be specified. This is then used recursively.!

!NeoJSONPropertyMapping methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/6/2012 15:34'!
getter: block
	getter := block! !

!NeoJSONPropertyMapping methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 7/4/2013 19:07'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(; print: propertyName; nextPut: $)! !

!NeoJSONPropertyMapping methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/6/2012 15:34'!
propertyName: string
	propertyName := string! !

!NeoJSONPropertyMapping methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 6/8/2012 17:04'!
readObject: anObject from: jsonReader
	| value |
	value := jsonReader nextAs: valueSchema.
	setter value: anObject value: value! !

!NeoJSONPropertyMapping methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/6/2012 15:34'!
setter: block
	setter := block! !

!NeoJSONPropertyMapping methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/8/2012 17:01'!
valueSchema: schemaName
	valueSchema := schemaName! !

!NeoJSONPropertyMapping methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 5/28/2014 20:46'!
writeObject: anObject on: jsonMapWriter
	| value |
	value := getter value: anObject.
	(value notNil or: [ jsonMapWriter writeNil ])
		ifTrue: [ jsonMapWriter writeKey: propertyName value: value as: valueSchema ] ! !

Object subclass: #NeoJSONStreamingWriter
	instanceVariableNames: 'jsonWriter first'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Core'!

!NeoJSONStreamingWriter commentStamp: '<historical>' prior: 0!
I am NeoJSONStreamingWriter.
I help in writing streamed maps and lists.

This is an internal class used by NeoJSONWriter to deal with interleaving elements and separators as well as pretty printing.!

!NeoJSONStreamingWriter class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 6/6/2012 15:05'!
on: jsonWriter
	^ self new
		on: jsonWriter;
		yourself! !

!NeoJSONStreamingWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/6/2012 14:48'!
initialize
	super initialize.
	first := true! !

!NeoJSONStreamingWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/6/2012 14:49'!
on: aJSONWriter
	jsonWriter := aJSONWriter! !

!NeoJSONStreamingWriter methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/6/2012 15:00'!
wasUsed
	^ first not! !

!NeoJSONStreamingWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 6/6/2012 15:15'!
writeElement: value
	first 
		ifTrue: [ 
			jsonWriter newlineIndent. 
			first := false ]
		ifFalse: [ 
			jsonWriter listElementSeparator ].
	jsonWriter nextPut: value ! !

!NeoJSONStreamingWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 6/6/2012 15:01'!
writeKey: key value: value
	first 
		ifTrue: [ 
			jsonWriter newlineIndent. 
			first := false ]
		ifFalse: [ 
			jsonWriter mapElementSeparator ].
	jsonWriter encodeKey: key value: value ! !

!NeoJSONStreamingWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 3/23/2014 21:53'!
writeKey: key value: value as: valueSchema
	first 
		ifTrue: [ 
			jsonWriter newlineIndent. 
			first := false ]
		ifFalse: [ 
			jsonWriter mapElementSeparator ].
	jsonWriter encodeKey: key value: value as: valueSchema! !

!NeoJSONStreamingWriter methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/28/2014 20:45'!
writeNil
	^ jsonWriter writeNil! !

!Object methodsFor: '*neo-json-core' stamp: 'SvenVanCaekenberghe 6/5/2012 22:29'!
neoJsonOn: neoJSONWriter
	neoJSONWriter writeObject: self! !

Error subclass: #NeoJSONParseError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Core'!

!NeoJSONParseError commentStamp: '<historical>' prior: 0!
I am NeoJSONParseError.
I am an Error.

I am signalled when JSON input is not valid while parsing.!

!String methodsFor: '*neo-json-core' stamp: 'SvenVanCaekenberghe 6/2/2012 22:26'!
neoJsonOn: neoJSONWriter
	neoJSONWriter writeString: self! !
